/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Feed Writer.
 *
 * The Initial Developer of the Original Code is Google Inc.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Ben Goodger <beng@google.com>
 *   Jeff Walden <jwalden+code@mit.edu>
 *   Asaf Romano <mano@mozilla.com>
 *   Robert Sayre <sayrer@gmail.com>
 *   nanto_vi (TOYAMA Nao) <nanto@moon.email.ne.jp>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function EFPNotification(aFeedResult) {
  this.feedResult = aFeedResult;
  this.feedURI = aFeedResult.uri.spec;
  gBrowser.addEventListener("load", this, true);
}
EFPNotification.popupCount = 0;

EFPNotification.prototype = {
  feedResult: null,
  feedURI: null,

  handleEvent: function EFPN_handleEvent(aEvent) {
    switch (aEvent.type) {
      case "load":
        if (aEvent.originalTarget.documentURI == this.feedURI) {
          var browser = gBrowser.getBrowserForDocument(aEvent.originalTarget);
          if (browser) {
            this.browser = browser;
            this.show();
          }
          gBrowser.removeEventListener("load", this, true);
        }
        break;

      case "command":
        if (aEvent.target.hasAttribute("efpCommand")) {
          switch (aEvent.target.getAttribute("efpCommand")) {
            case "never-show":
              gEFPFeedList.addItem(this.feedURI);
              this.notification.close();
              return;
            case "choose-app":
              // Show the file picker and subscribe the feed with
              // the selected application
              if (!this.chooseClientApp()) return;
              break;
          }
        } else {
          var handlerType = aEvent.target.getAttribute("efpHandlerType");
          this.handlerType = handlerType;
          if (handlerType == "web")
            this.handlerURI = aEvent.target.getAttribute("efpHandlerURI");
          else if (handlerType == "client")
            this.handlerFile = aEvent.target.efpFile;
        }

        this.subscribe();
        this.notification.close();
        break;

      case "pagehide":
        this.dispose();
        break;
    }
  },

  browser: null,
  popup: null,
  buttons: null,
  notification: null,

  // Show the notification message for the feed subscription
  show: function EFPN_show() {
    this.createPopup();
    this.buttons = this.createButtons();
    var notificationBox = gBrowser.getNotificationBox(this.browser);
    var efpBundle = document.getElementById("enhancedfeedpreview-bundle");
    this.notification =
      notificationBox.appendNotification(
        efpBundle.getString("subscribeFeed"), "efp-subscribe-feed",
        "chrome://browser/skin/page-livemarks.png",
        notificationBox.PRIORITY_INFO_MEDIUM, this.buttons);
    this.browser.addEventListener("pagehide", this, true);
  },

  // XXX Should we use nsIXULRuntime.OS?
  isWindows: /^Win/i.test(navigator.platform),
  isMacOS: /^Mac/i.test(navigator.platform),

  // Create popup menu for the "Advanced" button
  createPopup: function EFPN_createPopup() {
    var popup = document.createElementNS(kXULNS, "popup");
    popup.id = "enhancedfeedpreview-popup-" + EFPNotification.popupCount++;

    var efpBundle = document.getElementById("enhancedfeedpreview-bundle");
    var subscribeBundle =
      document.getElementById("enhancedfeedpreview-subscribe");

    // Live Bookmarks
    var menu = document.createElementNS(kXULNS, "menuitem");
    menu.className = "menuitem-iconic";
    var label = efpBundle.getFormattedString("subscribeWith",
                  [subscribeBundle.getString("liveBookmarks")]);
    menu.setAttribute("label", label);
    menu.setAttribute("image", "chrome://browser/skin/page-livemarks.png");
    menu.setAttribute("efpHandlerType", "bookmarks");
    popup.appendChild(menu);
    popup.appendChild(document.createElementNS(kXULNS, "menuseparator"));

    // Last-selected application
    var prefs = Cc["@mozilla.org/preferences-service;1"].
                getService(Ci.nsIPrefBranch);
    var app = null;
    try {
      app = prefs.getComplexValue("browser.feeds.handlers.application",
                                  Ci.nsILocalFile);
      if (app.exists()) {
        menu = document.createElementNS(kXULNS, "menuitem");
        this.initMenuWithFile(menu, app);
        popup.appendChild(menu);
      }
    } catch (e) {
    }

    // Default feed reader
    if (this.isWindows) {
      try {
        var regKey = Cc["@mozilla.org/windows-registry-key;1"].
                     createInstance(Ci.nsIWindowsRegKey);
        regKey.open(Ci.nsIWindowsRegKey.ROOT_KEY_CLASSES_ROOT,
                    "feed\\shell\\open\\command",
                    Ci.nsIWindowsRegKey.ACCESS_READ);
        var path = regKey.readStringValue("");
        regKey.close();
        if (path.charAt(0) == "\"")
          path = path.substring(1, path.indexOf("\"", 1));
        else if (path.indexOf(" ") != -1)
          path = path.substring(0, path.indexOf(" "));

        var defaultApp = Cc["@mozilla.org/file/local;1"].
                         createInstance(Ci.nsILocalFile);
        defaultApp.initWithPath(path);
        // Don't append the menu if the last-selected application is
        // same as the default application.
        if (defaultApp.exists() && (!app || app.path != path)) {
          menu = document.createElementNS(kXULNS, "menuitem");
          this.initMenuWithFile(menu, defaultApp);
          popup.appendChild(menu);
        }
      } catch (e) {
      }
    }

    // "Choose Application..." menuitem
    menu = document.createElementNS(kXULNS, "menuitem");
    label = subscribeBundle.getString("chooseApplicationMenuItem");
    menu.setAttribute("label", label);
    menu.setAttribute("efpCommand", "choose-app");
    popup.appendChild(menu);
    popup.appendChild(document.createElementNS(kXULNS, "menuseparator"));

    // List of web handlers
    var wccr =
      Cc["@mozilla.org/embeddor.implemented/web-content-handler-registrar;1"].
      getService(Ci.nsIWebContentConverterService);
    var handlers =
      wccr.getContentHandlers("application/vnd.mozilla.maybe.feed", {});
    var ios = Cc["@mozilla.org/network/io-service;1"].
              getService(Ci.nsIIOService);
    for (var i = 0; i < handlers.length; i++) {
      menu = document.createElementNS(kXULNS, "menuitem");
      menu.className = "menuitem-iconic";
      label = efpBundle.getFormattedString("subscribeWith", [handlers[i].name]);
      menu.setAttribute("label", label);
      menu.setAttribute("efpHandlerType", "web");
      menu.setAttribute("efpHandlerURI", handlers[i].uri);
      popup.appendChild(menu);

      // For privacy reasons we cannot set the image attribute directly
      // to the icon url, see Bug 358878
      try {
        var uri = ios.newURI(handlers[i].uri, null, null);
        if (/^https?/.test(uri.scheme))
          new EFPIconDataURIGenerator(uri.prePath + "/favicon.ico", menu);
      } catch (e) {
      }
    }
    if (handlers.length != 0)
      popup.appendChild(document.createElementNS(kXULNS, "menuseparator"));

    // "Never show the subscription message..." menuitem
    menu = document.createElementNS(kXULNS, "menuitem");
    menu.setAttribute("label", efpBundle.getString("neverShow"));
    menu.setAttribute("efpCommand", "never-show");
    popup.appendChild(menu);

    popup.addEventListener("command", this, false);
    document.getElementById("mainPopupSet").appendChild(popup);
    this.popup = popup;
  },

  initMenuWithFile: function EFPN_initMenuWithFile(aMenu, aFile) {
    var appName = "";
    if (this.isWindows) {
      if (aFile instanceof Ci.nsILocalFileWin) {
        try {
          appName = aFile.getVersionInfoField("FileDescription");
        } catch (e) {
        }
      }
    }
    if (this.isMacOS) {
      var lfm = aFile.QueryInterface(Ci.nsILocalFileMac_MOZILLA_1_8_BRANCH ||
                                     Ci.nsILocalFileMac);
      try {
        appName = lfm.bundleDisplayName;
      } catch (e) {
      }
    }
    var ios = Cc["@mozilla.org/network/io-service;1"].
              getService(Ci.nsIIOService);
    if (!appName) {
      var url = ios.newFileURI(aFile).QueryInterface(nsIURL);
      appName = url.fileName;
    }
    var label = document.getElementById("enhancedfeedpreview-bundle")
                        .getFormattedString("subscribeWith", [appName]);
    aMenu.setAttribute("label", label);

    var fph = ios.getProtocolHandler("file")
                 .QueryInterface(Ci.nsIFileProtocolHandler);
    var urlSpec = fph.getURLSpecFromFile(aFile);
    aMenu.className = "menuitem-iconic";
    aMenu.setAttribute("image", "moz-icon://" + urlSpec + "?size=16");

    aMenu.setAttribute("efpHandlerType", "client");
    aMenu.setAttribute("efpHandlerPath", aFile.path);
    aMenu.efpFile = aFile;
  },

  handlerType: null,
  handlerFile: null,
  handlerURI: null,

  // Create buttons' information for the notification message
  createButtons: function EFPN_createButtons() {
    var prefs = Cc["@mozilla.org/preferences-service;1"].
                getService(Ci.nsIPrefBranch);
    var handlerType = "bookmarks";
    try {
      handlerType = prefs.getCharPref("browser.feeds.handler.default");
    } catch (e) {
    }
    this.handlerType = handlerType;

    var defaultMenu = null;
    var menus = null;
    switch (handlerType) {
      case "web":
        var uri = prefs.getComplexValue("browser.feeds.handlers.webservice",
                                        Ci.nsISupportsString).data;
        menus = this.popup.getElementsByAttribute("efpHandlerURI", uri);
        if (menus.length != 0) {
          this.handlerURI = uri;
          defaultMenu = menus[0];
        }
        break;
      case "client":
        var app = null;
        try {
          app = prefs.getComplexValue("browser.feeds.handlers.application",
                                      Ci.nsILocalFile);
        } catch (e) {
          break;
        }
        menus = this.popup.getElementsByAttribute("efpHandlerPath", app.path);
        if (menus.length != 0) {
          this.handlerFile = app;
          defaultMenu = menus[0];
        }
        break;
    }
    if (!defaultMenu)
      // The default menuitem is the Live Bookmarks menuitem.
      defaultMenu = this.popup.firstChild;

    var efpBundles = document.getElementById("enhancedfeedpreview-bundle");
    return [
      {
        label: defaultMenu.getAttribute("label"),
        accessKey: efpBundles.getString("defaultHandlerAccessKey"),
        callback: this.createCallback()
      },
      {
        label: efpBundles.getString("advancedLabel"),
        accessKey: efpBundles.getString("advancedAccessKey"),
        popup: this.popup.id
      }
    ];
  },

  // Create the callback function for the default handler button
  createCallback: function EFPN_createCallback() {
    var self = this;
    function subscribe() {
      self.subscribe();
      return false;
    }
    return subscribe;
  },

  chooseClientApp: function EFPN_chooseClientApp() {
    try {
      var fp = Cc["@mozilla.org/filepicker;1"].
               createInstance(Ci.nsIFilePicker);
      fp.init(this.browser.contentWindow, "Choose app",
              Ci.nsIFilePicker.modeOpen);
      fp.appendFilters(Ci.nsIFilePicker.filterApps);

      if (fp.show() == Ci.nsIFilePicker.returnOK && fp.file) {
        // XXXben - we need to compare this with the running instance executable
        //          just don't know how to do that via script...
        // XXXmano TBD: can probably add this to nsIShellService
        if (this.isWindows) {
          if (fp.file.leafName == "firefox.exe") return false;
        } else if (this.isMacOS) {
          if (fp.file.leafName == "Firefox.app") return false;
        } else {
          if (fp.file.leafName == "firefox-bin") return false;
        }

        this.handlerType = "client";
        this.handlerFile = fp.file;
        return true;
      }
    } catch (e) {
    }
    return false;
  },

  // Subscribe to the feed using the selected handler and save prefs
  subscribe: function EFPN_subscript() {
    var prefs = Cc["@mozilla.org/preferences-service;1"].
                getService(Ci.nsIPrefBranch);
    prefs.setCharPref("browser.feeds.handler.default", this.handlerType);

    if (this.handlerType == "web") {
      var string = Cc["@mozilla.org/supports-string;1"].
                   createInstance(Ci.nsISupportsString);
      string.data = this.handlerURI;
      prefs.setComplexValue("browser.feeds.handlers.webservice",
                            Ci.nsISupportsString, string);

      var wccr =
        Cc["@mozilla.org/embeddor.implemented/web-content-handler-registrar;1"].
        getService(Ci.nsIWebContentConverterService);
      var handler =
        wccr.getWebContentHandlerByURI("application/vnd.mozilla.maybe.feed",
                                       this.handlerURI);
      if (handler)
        this.browser.loadURI(handler.getHandlerURI(this.feedURI));
    } else {
      if (this.handlerType == "client")
        prefs.setComplexValue("browser.feeds.handlers.application",
                              Ci.nsILocalFile, this.handlerFile);

      var feedService = Cc["@mozilla.org/browser/feeds/result-service;1"].
                        getService(Ci.nsIFeedResultService);
      var feedTitle = this.feedResult.doc.title.plainText();
      var feedSubtitle = this.feedResult.doc.QueryInterface(Ci.nsIFeed)
                             .subtitle.plainText();
      feedService.addToClientReader(this.feedURI, feedTitle, feedSubtitle);
    }

    // Append the feed URI to the list of feeds for which we don't show
    // the subscription message.
    gEFPFeedList.addItem(this.feedURI);
  },

  dispose: function EFPN_dispose() {
    this.browser.removeEventListener("pagehide", this, true);
    this.popup.parentNode.removeChild(this.popup);
    this.buttons[0].callback = null;
    this.feedResult = null;
    this.browser = null;
    this.popup = null;
    this.notification = null;
  }
};


// FeedWriter.js iconDataURIGenerator
function EFPIconDataURIGenerator(aURISpec, aElement) {
  var ios = Cc["@mozilla.org/network/io-service;1"].
            getService(Ci.nsIIOService);
  // Change from FeedWriter.js
  // var chan = ios.newChannelFromURI(makeURI(aURISpec));
  var chan = ios.newChannel(aURISpec, null, null);
  chan.notificationCallbacks = this;
  chan.asyncOpen(this, null);

  this._channel = chan;
  this._bytes = [];
  this._element = aElement;
}
EFPIconDataURIGenerator.prototype = {
  _channel: null,
  _countRead: 0,
  _stream: null,

  QueryInterface: function FW_IDUG_loadQI(aIID) {
    if (aIID.equals(Ci.nsISupports)           ||
        aIID.equals(Ci.nsIRequestObserver)    ||
        aIID.equals(Ci.nsIStreamListener)     ||
        aIID.equals(Ci.nsIChannelEventSink)   ||
        aIID.equals(Ci.nsIInterfaceRequestor) ||
        aIID.equals(Ci.nsIBadCertListener)    ||
        // See bug 358878 comment 11
        aIID.equals(Ci.nsIPrompt)             ||
        // See FIXME comment below
        aIID.equals(Ci.nsIHttpEventSink)      ||
        aIID.equals(Ci.nsIProgressEventSink)  ||
        false)
      return this;

    throw Cr.NS_ERROR_NO_INTERFACE;
  },

  // nsIRequestObserver
  onStartRequest: function FW_IDUG_loadStartR(aRequest, aContext) {
    this._stream = Cc["@mozilla.org/binaryinputstream;1"].
                   createInstance(Ci.nsIBinaryInputStream);
  },

  onStopRequest: function FW_IDUG_loadStopR(aRequest, aContext, aStatusCode) {
    var requestFailed = !Components.isSuccessCode(aStatusCode);
    if (!requestFailed && (aRequest instanceof Ci.nsIHttpChannel))
      requestFailed = !aRequest.requestSucceeded;

    if (!requestFailed && this._countRead != 0) {
      var str = String.fromCharCode.apply(null, this._bytes);
      try {
        // Change from FeedWriter.js
        // var dataURI = ICON_DATAURL_PREFIX + btoa(str);
        var dataURI = "data:image/x-icon;base64," + btoa(str);
        this._element.setAttribute("image", dataURI);
      }
      catch(ex) {}
    }
    this._channel = null;
    this._element  = null;
  },

  // nsIStreamListener
  onDataAvailable: function FW_IDUG_loadDAvailable(aRequest, aContext,
                                                   aInputStream, aOffset,
                                                   aCount) {
    this._stream.setInputStream(aInputStream);

    // Get a byte array of the data
    this._bytes = this._bytes.concat(this._stream.readByteArray(aCount));
    this._countRead += aCount;
  },

  // nsIChannelEventSink
  onChannelRedirect: function FW_IDUG_loadCRedirect(aOldChannel, aNewChannel,
                                                    aFlags) {
    this._channel = aNewChannel;
  },

  // nsIInterfaceRequestor
  getInterface: function FW_IDUG_load_GI(aIID) {
    return this.QueryInterface(aIID);
  },

  // nsIBadCertListener
  confirmUnknownIssuer: function FW_IDUG_load_CUI(aSocketInfo, aCert,
                                                  aCertAddType) {
    return false;
  },

  confirmMismatchDomain: function FW_IDUG_load_CMD(aSocketInfo, aTargetURL,
                                                   aCert) {
    return false;
  },

  confirmCertExpired: function FW_IDUG_load_CCE(aSocketInfo, aCert) {
    return false;
  },

  notifyCrlNextupdate: function FW_IDUG_load_NCN(aSocketInfo, aTargetURL, aCert) {
  },

  // FIXME: bug 253127
  // nsIHttpEventSink
  onRedirect: function (aChannel, aNewChannel) { },
  // nsIProgressEventSink
  onProgress: function (aRequest, aContext, aProgress, aProgressMax) { },
  onStatus: function (aRequest, aContext, aStatus, aStatusArg) { }
};
