const Cc = Components.classes;
const Ci = Components.interfaces;

var gEFPPreferences = {
  feedList: null,

  init: function EFPP_init() {
    var feedList = gEFPFeedList.getList();
    var listBox = document.getElementById("feedlist-list");

    for (var i = 0; i < feedList.length; i++) {
      var listItem = listBox.appendItem(feedList[i]);
      listItem.tooltipText = feedList[i];
    }

    this.feedList = feedList;
    this.updateRemoveButtons(listBox);
    document.getElementById("feedlist-uri").focus();
  },

  updateRemoveButtons: function EFPP_updateRemoveButtons(aListBox) {
    document.getElementById("feedlist-remove").disabled =
      !aListBox.selectedCount;
    document.getElementById("feedlist-removeall").disabled =
      !aListBox.hasChildNodes();
  },

  onDialogAccept: function EFPP_onDialogAccept() {
    gEFPFeedList.setList(this.feedList);
  },
  onURIInput: function EFPP_onURIInput(aEvent) {
    document.getElementById("feedlist-add").disabled = !aEvent.target.value;
  },
  onURIKeyPress: function EFPP_onURIKeyPress(aEvent) {
    if (aEvent.keyCode == KeyEvent.DOM_VK_RETURN) {
      this.addFeedURI();
      aEvent.preventDefault();
    }
  },
  onListKeyPress: function EFPP_onListKeyPress(aEvent) {
    switch (aEvent.keyCode) {
      case KeyEvent.DOM_VK_DELETE:
        this.removeFeedURI();
        break;
      case KeyEvent.DOM_VK_RETURN:
        aEvent.preventDefault();
        break;
    }
  },

  addFeedURI: function EFPP_addFeedURI() {
    var textBox = document.getElementById("feedlist-uri");
    var feedURI = textBox.value;
    if (!feedURI) {
      textBox.focus();
      return;
    }

    var index = gEFPFeedList.getInsertionIndex(this.feedList, feedURI);
    var listBox = document.getElementById("feedlist-list");
    var listItem;

    if (index == -1) {
      listItem = listBox.getElementsByAttribute("label", feedURI)[0];
    } else {
      listItem = (index == this.feedList.length)
                 ? listBox.appendItem(feedURI)
                 : listBox.insertItemAt(index, feedURI);
      listItem.tooltipText = feedURI;
      this.feedList.splice(index, 0, feedURI);
    }

    listBox.selectItem(listItem);
    listBox.ensureElementIsVisible(listItem);
    textBox.value = "";
    textBox.focus();
  },

  removeFeedURI: function EFPP_removeFeedURI() {
    var listBox = document.getElementById("feedlist-list");
    // Copy selectedItems because clearSelection() also clears selectedItems
    var selectedItems = listBox.selectedItems.concat();

    listBox.clearSelection();
    for (var i = 0; i < selectedItems.length; i++) {
      var index = listBox.getIndexOfItem(selectedItems[i]);
      listBox.removeItemAt(index);
      this.feedList.splice(index, 1);
    }

    this.updateRemoveButtons(listBox);
  },
  removeAllFeedURIs: function EFPP_removeAllFeedURIs() {
    var listBox = document.getElementById("feedlist-list");
    listBox.clearSelection();

    var range = document.createRange();
    range.selectNodeContents(listBox);
    range.deleteContents();
    this.feedList.length = 0;

    this.updateRemoveButtons(listBox);
  }
};
